@@ -41,7 +41,7 @@
 #include <devstruct.h>
 #include <devices.h>
 
-#define RTAILAB_VERSION   "3.1.1"
+#define RTAILAB_VERSION   "3.2.1"
 #define MAX_ADR_SRCH      500
 #define MAX_NAME_SIZE     256
 #define MAX_SCOPES        100
@@ -116,6 +116,8 @@ int ComediDev_DIOInUse[MAX_COMEDI_DEVICES] = {0};
 
 devStr inpDevStr[40];
 devStr outDevStr[40];
+int pinp_cnt = 0;
+int pout_cnt = 0;
 
 static void DummyWait(void) { }
 static void DummySend(void) { }
@@ -494,6 +496,7 @@ static void *rt_HostInterface(void *args)
 		int i, j, Idx;
 		rtTargetParamInfo rtParam;
 
+		strncpyz(rtParam.modelName, STR(MODEL), MAX_NAME_SIZE);
 		rtParam.dataType  = SS_DOUBLE;
 		rtParam.dataClass = rt_SCALAR;
    	        rtParam.nRows = 1;
@@ -753,6 +756,8 @@ static void endme(int dummy)
   signal(SIGINT, endme);
   signal(SIGTERM, endme);
   endex = 1;
+  endBaseRate=1;
+  endInterface=1;
 }
 
 void exit_on_error()
