@@ -53,11 +53,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 #include <efltk/fl_ask.h>
 #include <efltk/Fl_Text_Editor.h>
 
-// #define KEEP_STATIC_INLINE
-// #include <rtai_lxrt_user.h>
-// #include <rtai_lxrt.h>
-// #include <net_rpc.h>
-
 #include <rtai_netrpc.h>
 #include <rtai_msg.h>
 #include <rtai_mbx.h>
@@ -767,6 +762,7 @@ void rlg_batch_update_parameters_cb(Fl_Widget *o, void *v)
 	}
 	if (Parameters_Manager->batch_counter() > 0 && n == Parameters_Manager->batch_counter()) {
 		RT_RPC(Target_Interface_Task, BATCH_DOWNLOAD, 0);
+		Parameters_Manager->batch_counter( 0);
 	}
 }
 
@@ -1471,8 +1467,8 @@ double get_parameter(Target_Parameters_T p, int nr, int nc, int *val_idx)
 			*val_idx = 0;
 			return (p.data_value[0]);
 		case rt_VECTOR:
-			*val_idx = nr;
-			return (p.data_value[nr]);
+			*val_idx = nc;
+			return (p.data_value[nc]);
 		case rt_MATRIX_ROW_MAJOR:
 			*val_idx = nr*p.n_cols+nc;
 			return (p.data_value[nr*p.n_cols+nc]);
